/*
 * 쐬: 2005/08/26
 *
 */
package org.seasar.buri.engine.invoker.impl;

import java.util.Date;
import java.util.List;

import org.seasar.buri.dao.BuriPathDao;
import org.seasar.buri.dao.BuriStateDao;
import org.seasar.buri.dto.FurnitureItemDto;
import org.seasar.buri.engine.BuriEngine;
import org.seasar.buri.engine.ParticipantProvider;
import org.seasar.buri.engine.invoker.SimpleBuriInvoker;
import org.seasar.buri.engine.invoker.StandardBuriInvoker;
import org.seasar.extension.unit.S2TestCase;
import org.seasar.framework.log.Logger;


public class StandardBuriInvokerTest extends S2TestCase {
    private String PATH = "org/seasar/buri/engine/invoker/impl/StandardBuriInvoker.dicon";
    private static Logger logger = Logger.getLogger(StandardBuriInvokerTest.class);
    private BuriEngine buriEngine;
    private StandardBuriInvoker invoker_;
    private SimpleBuriInvoker simpleInvoker_;
    
    private BuriStateDao stateDao_;
    private BuriPathDao pathDao_;
    
    private final String roleSato = ",ے,\";
    private final String roleSuzuki = ",ے㗝,\";
    private final String roleTakahashi = ",W,\";
    private final String roleTanaka = "c,C,\";
    private final String roleWatanabe = "n,ʎЈ,\";
    private final String roleIto = "ɓ,ʎЈ,\";
    private final String roleYamamoto = "R{,ʎЈ,\";
    
    public StandardBuriInvokerTest(String arg0) {
        super(arg0);
    }
    
    protected void setUp() throws Exception {
        super.setUp();
        include(PATH);
        ParticipantProvider provider = (ParticipantProvider)getComponent(ParticipantProvider.class);
        buriEngine = (BuriEngine)getComponent(BuriEngine.class);
//        buriEngine.getWorkflows().readWorkFlowFromResource("YǗ","petition.xpdl");
        buriEngine.getWorkflows().readWorkFlowFromResource("YǗ","petition.xpdl",provider);
        
    }
    
    public void testStandardTestTx() {
        FurnitureItemDto buyItemDto = new FurnitureItemDto();
        buyItemDto.setName("T45_001");
        buyItemDto.setType("PC");
        buyItemDto.setAcquisitionTypeBuy();
        buyItemDto.setAcquisition(new Date());
        List datas;
        
        // ʎЈF\葱Jn
        logger.debug("******************************");
        logger.debug("invoke YǗ.w\");
        invoker_.invoke("YǗ.w\",buyItemDto,roleYamamoto);
        
        // \
        logger.debug("******************************");
        logger.debug("getDataListFromPathAndUser YǗ.w\.\ "+roleYamamoto);
        datas = invoker_.getDataListFromPath("YǗ.w\.\",roleYamamoto);
        assertEquals(datas.size(),1);
        logger.debug(datas);
        
        // CFF҂
        logger.debug("******************************");
        logger.debug("getDataListFromPath YǗ.w\.F҂ "+roleTanaka);
        datas = invoker_.getDataListFromPath("YǗ.w\.F҂",roleTanaka);
        assertEquals(datas.size(),1);
        logger.debug(datas);
        
        logger.debug(pathDao_.getAllBuriPath());
        logger.debug(stateDao_.getAllBuriState());
        logger.debug(stateDao_.getNoProcessBuriState());
        
        // CFF
        logger.debug("******************************");
        logger.debug("invokeAction YǗ.w\ "+roleTanaka);
        invoker_.invokeAction("YǗ.w\",buyItemDto ,"approve",roleTanaka);
        
        logger.debug(pathDao_.getAllBuriPath());
        logger.debug(stateDao_.getAllBuriState());
        logger.debug(stateDao_.getNoProcessBuriState());
        
        // CFFς
        logger.debug("******************************");
        logger.debug("getDataListFromPath YǗ.w\.F҂ "+roleTanaka);
        datas = invoker_.getDataListFromPath("YǗ.w\.F҂",roleTanaka);
        assertEquals(datas.size(),0);

        // WFF҂
        logger.debug("******************************");
        logger.debug("getDataListFromPath YǗ.w\.F҂ "+roleTakahashi);
        datas = invoker_.getDataListFromPath("YǗ.w\.F҂",roleTakahashi);
        assertEquals(datas.size(),1);
        logger.debug(datas);
        
        // \
        logger.debug("******************************");
        logger.debug("getDataListFromPathAndUser YǗ.w\.\ "+roleYamamoto);
        datas = invoker_.getDataListFromPath("YǗ.w\.\",roleYamamoto);
        assertEquals(datas.size(),1);


        // WFF
        logger.debug("******************************");
        logger.debug("invokeAction YǗ.w\ "+roleTakahashi);
        invoker_.invokeAction("YǗ.w\",buyItemDto ,"approve",roleTakahashi);
        
        // WFFς
        logger.debug("******************************");
        logger.debug("getDataListFromPath YǗ.w\.F҂ "+roleTakahashi);
        datas = invoker_.getDataListFromPath("YǗ.w\.F҂",roleTakahashi);
        assertEquals(datas.size(),0);
        
        // ے㗝FF҂
        logger.debug("******************************");
        logger.debug("getDataListFromPath YǗ.w\.F҂ "+roleSuzuki);
        datas = invoker_.getDataListFromPath("YǗ.w\.F҂",roleSuzuki);
        assertEquals(datas.size(),1);
        logger.debug(datas);
        
        // ےFF҂
        logger.debug("******************************");
        logger.debug("getDataListFromPath YǗ.w\.F҂ "+roleSato);
        datas = invoker_.getDataListFromPath("YǗ.w\.F҂",roleSato);
        assertEquals(datas.size(),1);
        logger.debug(datas);
        
        // \
        logger.debug("******************************");
        logger.debug("getDataListFromPathAndUser YǗ.w\.\ "+roleYamamoto);
        datas = invoker_.getDataListFromPath("YǗ.w\.\",roleYamamoto);
        assertEquals(datas.size(),1);
        logger.debug(datas);
        
        // ےFF
        logger.debug("******************************");
        logger.debug("invokeAction YǗ.w\ "+roleSato);
        invoker_.invokeAction("YǗ.w\",buyItemDto ,"approve",roleSato);

        
        // ے㗝FFς
        logger.debug("******************************");
        logger.debug("getDataListFromPath YǗ.w\.F҂ "+roleSuzuki);
        datas = invoker_.getDataListFromPath("YǗ.w\.F҂",roleSuzuki);
        assertEquals(datas.size(),0);
        
        // ےFFς
        logger.debug("******************************");
        logger.debug("getDataListFromPath YǗ.w\.F҂ "+roleSato);
        datas = invoker_.getDataListFromPath("YǗ.w\.F҂",roleSato);
        assertEquals(datas.size(),0);
        

        // \ς
        logger.debug("******************************");
        logger.debug("getDataListFromPathAndUser YǗ.w\.\ "+roleYamamoto);
        datas = invoker_.getDataListFromPath("YǗ.w\.\",roleYamamoto);
        assertEquals(datas.size(),0);
   

        // Fς
        logger.debug("******************************");
        logger.debug("getDataListFromPathAndUser YǗ.w\.Fς "+roleYamamoto);
        datas = invoker_.getDataListFromPath("YǗ.w\.Fς",roleYamamoto);
        assertEquals(datas.size(),1);
        logger.debug(datas);
    
    
    }

    public void testReturningTestTx() {
        FurnitureItemDto buyItemDto = new FurnitureItemDto();
        buyItemDto.setName("T45_001");
        buyItemDto.setType("PC");
        buyItemDto.setAcquisitionTypeBuy();
        buyItemDto.setAcquisition(new Date());
        List datas;
        
        // ʎЈF\葱Jn
        logger.debug("******************************");
        logger.debug("invoke YǗ.w\");
        invoker_.invoke("YǗ.w\",buyItemDto,roleYamamoto);
        
        // \
        logger.debug("******************************");
        logger.debug("getDataListFromPathAndUser YǗ.w\.\");
        datas = invoker_.getDataListFromPath("YǗ.w\.\",roleYamamoto);
        assertEquals(datas.size(),1);
        logger.debug(datas);
        
        // CFF҂
        logger.debug("******************************");
        logger.debug("getDataListFromPath YǗ.w\.F҂");
        datas = invoker_.getDataListFromPath("YǗ.w\.F҂",roleTanaka);
        assertEquals(datas.size(),1);
        logger.debug(datas);
        
        // CF߂
        logger.debug("******************************");
        logger.debug("invokeAction YǗ.w\");
        invoker_.invokeAction("YǗ.w\",buyItemDto ,"returning",roleTanaka);
        
        // ʎЈF߂ς
        logger.debug("******************************");
        logger.debug("getDataListFromPath YǗ.w\.߂ς");
        datas = invoker_.getDataListFromPath("YǗ.w\.߂ς",roleYamamoto);
        assertEquals(datas.size(),1);
        logger.debug(datas);
        
        // \
        logger.debug("******************************");
        logger.debug("getDataListFromPathAndUser YǗ.w\.\");
        datas = invoker_.getDataListFromPath("YǗ.w\.\",roleYamamoto);
        assertEquals(datas.size(),0);

    
    }
    
    
    
}
